#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import print_function
from BaseHTTPServer import BaseHTTPRequestHandler, HTTPServer

from storage import Storage
import json
import logging
import re
from SocketServer import ThreadingMixIn

class BatcaveHttpRequestHandler(BaseHTTPRequestHandler):
	def parse_url_pathquery(self):
		url = re.match(r'^\/(?P<path>.+?)(\?(?P<query>.+))?$', self.path)
		if url is None:
			logging.warn('Failed to parse URL \'' + str(self.path) + '\'.')
			return ( None, None )

		path = url.group('path')
		query = {}
		if not url.group('query') is None:
			for m in re.finditer(r'(?P<key>.+?)=(?P<value>.+?)(&|$)', url.group('query')):
				query[m.group('key')] = m.group('value')
		return ( path, query )
		
	def do_GET(self):
		path, query = self.parse_url_pathquery()

		if path == '':
			self.respond_index(query)
			return

		if path == 'list':
			self.respond_list(query)
			return

		m = re.match(r'node/([a-f0-9]{12})(?P<cmd>\.json|/[a-zA-Z0-9_\-]+)$', path)
		if m != None:
			cmd = m.group('cmd')
			if cmd == '.json':
				self.respond_node(m.group(1))
			else:
				self.respond_nodedetail(m.group(1), cmd[1:])
			return

		self.send_error(404, 'The URL \'{0}\' was not found here.'.format(path))

	def send_nocache_headers(self):
		self.send_header('Cache-Control', 'no-cache, no-store, must-revalidate')
		self.send_header('Pragma', 'no-cache')
		self.send_header('Expires', '0')

	def respond_index(self, query):
		storagy = self.server.storage

		self.send_response(200)
		self.send_header('Content-Type', 'text/html; charset=utf-8')
		self.send_nocache_headers()
		self.end_headers()

		self.wfile.write('<!DOCTYPE html><html><head><title>BATCAVE</title></head>\n')
		self.wfile.write('<body>\n')
		self.wfile.write('<H1 title="Batman/Alfred Transmission Collection, Aggregation & Value Engine">BATCAVE</H1>\n')

		self.wfile.write('<p>Dies ist ein interner Hintergrund-Dienst. Er wird nur von anderen Diensten\n')
		self.wfile.write('angesprochen und sollte aus einer Mehrzahl von Gr&uuml;nden nicht &ouml;ffentlich\n')
		self.wfile.write('zug&auml;nglich sein.</p>\n')

		self.wfile.write('<H2>Status</H2>\n')
		self.wfile.write('Daten: <span id="datacount" class="value">')
		self.wfile.write(len(storage.data))
		self.wfile.write('</span>\n')

		self.wfile.write('<H2>API</H2>\n')
		self.wfile.write('<p>Grundsätzlich ist das Antwort-Format JSON und alle Daten sind Live-Daten (kein Cache) die ggf. etwas Bearbeitungs-Zeit erfordern.</p>')
		self.wfile.write('<dl>\n')
		self.wfile.write('<dt><a href="/nodes.json">nodes.json</a></dt><dd>zur Verwendung mit ffmap (MACs anonymisiert)</dd>\n')
		self.wfile.write('<dt><a href="/node/ff00ff00ff00.json">/node/&lt;id&gt;.json</a></dt><dd><u>alle</u> vorhandenen Information zu der gewünschten Node</dd>\n')
		self.wfile.write('</dl>\n')
		self.wfile.write('</body></html>')

	def respond_list(self, query):
		storage = self.server.storage
		self.send_response(200)
		self.send_header('Content-Type', 'text/html')
		self.send_nocache_headers()
		self.end_headers()

		self.wfile.write('<!DOCTYPE html><html><head><title>BATCAVE</title></head>\n')
		self.wfile.write('<body>\n')
		self.wfile.write('<H1>BATCAVE - LIST</H1>\n')

		self.wfile.write('<table>\n')
		self.wfile.write('<thead><tr><th>ID</th><th>Name</th></tr></thead>\n')
		self.wfile.write('<tbody>\n')

		data = storage.data
		if 'sort' in query:
			if query['sort'] == 'name':
				sorteddata = sorted(data, key=lambda x: data[x]['hostname'].lower())
				data = sorteddata
			elif query['sort'] == 'id':
				sorteddata = sorted(data)
				data = sorteddata

		for nodeid in data:
			nodename = storage.data[nodeid]['hostname'] if 'hostname' in storage.data[nodeid] else '&lt;?&gt;'
			self.wfile.write('<tr><td><a href="/node/' + nodeid + '.json">' + nodeid + '</a></td><td>' + nodename + '</td></tr>')

		self.wfile.write('</tbody>\n')
		self.wfile.write('</table>\n')

	def respond_node(self, nodeid):
		storage = self.server.storage

		if nodeid == 'ff00ff00ff00':
			self.send_response(200)
			self.send_header('Content-Type', 'text/json')
			self.send_nocache_headers()
			self.end_headers()
			self.wfile.write(json.dumps({
				'name': 'API-Example',
				'nodeid': nodeid,
				'META': 'Dies ist ein minimaler Beispiel-Datensatz. Herzlichen Glückwunsch, du hast das Prinzip der API kapiert.',
			}))
			return

		if not nodeid in storage.data:
			self.send_error(404, 'No node with id \'' + nodeid + '\' present.')
			return

		self.send_response(200)
		self.send_header('Content-Type', 'text/json')
		self.send_nocache_headers()
		self.end_headers()
		self.wfile.write(json.dumps(storage.data[nodeid]))

	def respond_nodedetail(self, nodeid, field):
		storage = self.server.storage

		if not nodeid in storage.data:
			self.send_error(404, 'No node with id \'' + nodeid + '\' present.')
			return

		if not field in storage.data[nodeid]:
			self.send_error(404, 'The node \'' + nodeid + '\' does not have a field named \'' + str(field) + '\'.')
			return

		value = storage.data[nodeid][field]

		self.send_response(200)
		self.send_header('Content-Type', 'text/plain' if isinstance(value, basestring) else 'text/json')
		self.send_nocache_headers()
		self.end_headers()
		self.wfile.write(value if isinstance(value, basestring) else json.dumps(value))

class ApiServer(ThreadingMixIn, HTTPServer):
	def __init__(self, endpoint, storage):
		HTTPServer.__init__(self, endpoint, BatcaveHttpRequestHandler)
		self.storage = storage

	def __str__(self):
		return 'ApiServer on {0}'.format(self.server_address)

if __name__ == '__main__':
	dummystorage = Storage()
	server = ApiServer(('0.0.0.0', 8888), dummystorage)

	print("Server:", str(server))
	server.serve_forever()
